@@ -661,9 +661,11 @@ class MSSMscolab(QtCore.QObject):
                     self.add_proj_dialog.buttonBox.button(QtWidgets.QDialogButtonBox.Ok).setEnabled(False)
 
             def browse():
-                file_type = ["Flight track (*.ftml)"] + [
-                    f"Flight track (*.{ext})" for ext in self.ui.import_plugins.keys()
-                ]
+                type = self.add_proj_dialog.cb_ImportType.currentText()
+                file_type = ["Flight track (*.ftml)"]
+                if type != 'FTML':
+                    file_type = [f"Flight track (*.{self.ui.import_plugins[type][1]})"]
+
                 file_path = get_open_filename(
                     self.ui, "Open Flighttrack file", "", ';;'.join(file_type))
                 if file_path is not None:
@@ -672,8 +674,7 @@ class MSSMscolab(QtCore.QObject):
                         with open_fs(fs.path.dirname(file_path)) as file_dir:
                             file_content = file_dir.readtext(file_name)
                     else:
-                        ext = fs.path.splitext(file_path)[-1][1:]
-                        function = self.ui.import_plugins[ext]
+                        function = self.ui.import_plugins[type][0]
                         ft_name, waypoints = function(file_path)
                         model = ft.WaypointsTableModel(waypoints=waypoints)
                         xml_doc = model.get_xml_doc()
@@ -692,6 +693,11 @@ class MSSMscolab(QtCore.QObject):
             self.add_proj_dialog.category.textChanged.connect(check_and_enable_operation_accept)
             self.add_proj_dialog.browse.clicked.connect(browse)
             self.add_proj_dialog.category.setText(config_loader(dataset="MSCOLAB_category"))
+
+            # sets types from defined import menu
+            import_menu = self.ui.menuImportFlightTrack
+            for im_action in import_menu.actions():
+                self.add_proj_dialog.cb_ImportType.addItem(im_action.text())
             self.proj_diag.show()
         else:
             show_popup(self.ui, "Error", "Your Connection is expired. New Login required!")
@@ -897,6 +903,7 @@ class MSSMscolab(QtCore.QObject):
                     try:
                         res = requests.post(url, data=data)
                         res.raise_for_status()
+                        self.reload_operations()
                     except requests.exceptions.RequestException as e:
                         logging.debug(e)
                         show_popup(self.ui, "Error", "Some error occurred! Could not delete operation.")
