@@ -50,17 +50,21 @@ import io
 import logging
 import traceback
 import urllib.parse
+import inspect
 from xml.etree import ElementTree
 from chameleon import PageTemplateLoader
 from owslib.crs import axisorder_yx
 from PIL import Image
+import shutil
 
 from flask import request, make_response, render_template
 from flask_httpauth import HTTPBasicAuth
 from multidict import CIMultiDict
 from mslib.utils import conditional_decorator
-from mslib.utils import parse_iso_datetime
+from mslib.utils.time import parse_iso_datetime
 from mslib.index import app_loader
+from mslib.mswms.gallery_builder import add_image, write_html, add_levels, add_times, \
+    write_doc_index, write_code_pages, STATIC_LOCATION, DOCS_LOCATION
 
 # Flask basic auth's documentation
 # https://flask-basicauth.readthedocs.io/en/latest/#flask.ext.basicauth.BasicAuth.check_credentials
@@ -129,7 +133,7 @@ if mss_wms_settings.__dict__.get('enable_basic_http_authentication', False):
         return authfunc(username, password)
 
 from mslib.mswms import mss_plot_driver
-from mslib.utils import get_projection_params
+from mslib.utils.coordinate import get_projection_params
 
 # Logging the Standard Output, which will be added to the Apache Log Files
 logging.basicConfig(level=logging.DEBUG,
@@ -205,10 +209,196 @@ class WMSServer(object):
             mss_wms_settings.register_linear_layers = []
         for layer in mss_wms_settings.register_linear_layers:
             if len(layer) == 3:
-                self.register_lsec_layer(layer[2], layer[1], layer[0])
+                self.register_lsec_layer(layer[2], layer[1], layer_class=layer[0])
+            elif len(layer) == 4:
+                self.register_lsec_layer(layer[3], layer[1], layer[2], layer[0])
             else:
                 self.register_lsec_layer(layer[1], layer_class=layer[0])
 
+    def generate_gallery(self, create=False, clear=False, generate_code=False, sphinx=False, plot_list=None,
+                         all_plots=False, url_prefix="", levels="", itimes="", vtimes="", simple_naming=False):
+        """
+        Iterates through all registered layers, draws their plots and puts them in the gallery
+        """
+        if mss_wms_settings.__file__:
+            if all_plots:
+                # Imports here due to some circular import issue if imported too soon
+                from mslib.mswms import mpl_hsec_styles, mpl_vsec_styles, mpl_lsec_styles
+
+                dataset = [next(iter(mss_wms_settings.data))]
+                mss_wms_settings.register_horizontal_layers = [
+                    (plot[1], dataset) for plot in inspect.getmembers(mpl_hsec_styles, inspect.isclass)
+                    if not any(x in plot[0] or x in str(plot[1]) for x in ["Abstract", "Target", "fnord"])
+                ]
+                mss_wms_settings.register_vertical_layers = [
+                    (plot[1], dataset) for plot in inspect.getmembers(mpl_vsec_styles, inspect.isclass)
+                    if not any(x in plot[0] or x in str(plot[1]) for x in ["Abstract", "Target", "fnord"])
+                ]
+                mss_wms_settings.register_linear_layers = [
+                    (plot[1], dataset) for plot in inspect.getmembers(mpl_lsec_styles, inspect.isclass)
+                ]
+                self.__init__()
+
+            location = DOCS_LOCATION if sphinx else STATIC_LOCATION
+            if clear and os.path.exists(os.path.join(location, "plots")):
+                shutil.rmtree(os.path.join(location, "plots"))
+            if os.path.exists(os.path.join(location, "code")):
+                shutil.rmtree(os.path.join(location, "code"))
+            if os.path.exists(os.path.join(location, "plots.html")):
+                os.remove(os.path.join(location, "plots.html"))
+
+            if not (create or generate_code or all_plots or plot_list):
+                return
+
+            if not plot_list:
+                plot_list = [[self.lsec_drivers, self.lsec_layer_registry],
+                             [self.vsec_drivers, self.vsec_layer_registry],
+                             [self.hsec_drivers, self.hsec_layer_registry]]
+
+            # Iterate through all plots of all datasets, create the plot and build the gallery with it
+            for driver, registry in plot_list:
+                multiple_datasets = len(driver) > 1
+                for dataset in driver:
+                    plot_driver = driver[dataset]
+                    if dataset not in registry:
+                        continue
+                    for plot in registry[dataset]:
+                        plot_object = registry[dataset][plot]
+                        l_type = "Linear" if driver == self.lsec_drivers else \
+                            "Side" if driver == self.vsec_drivers else "Top"
+
+                        try:
+                            file_types = [field[0] for field in plot_object.required_datafields
+                                          if field[0] != "sfc"]
+                            file_type = file_types[0] if file_types else "sfc"
+
+                            # All specified init times, or the latest if empty, or all if "all",
+                            # or None if there are no init times
+                            init_times = [parse_iso_datetime(itime) if isinstance(itime, str) else itime
+                                          for itime in (itimes.split(",") if itimes != "all" and itimes != "" else
+                                                        plot_driver.get_init_times() if itimes == "all" else
+                                                        [plot_driver.get_init_times()[-1]])] or [None]
+
+                            for itime in sorted(init_times):
+                                if itime and plot_driver.get_init_times() and itime not in plot_driver.get_init_times():
+                                    logging.warning(f"Requested itime {itime} not present for "
+                                                    f"{dataset} {plot_object.name}! itimes present: "
+                                                    f"{plot_driver.get_init_times()}")
+                                    continue
+                                elif not plot_driver.get_init_times():
+                                    itime = None
+
+                                # All valid times for the specific init time
+                                i_vtimes = plot_driver.get_valid_times(plot_object.required_datafields[0][1], file_type,
+                                                                       itime)
+
+                                # All specified valid times, or the latest if empty, or all if "all",
+                                # or None if there are no valid times for the init time
+                                valid_times = [parse_iso_datetime(vtime) if isinstance(vtime, str) else vtime
+                                               for vtime in (vtimes.split(",") if vtimes != "all" and vtimes != "" else
+                                                             i_vtimes if vtimes == "all" else [i_vtimes[-1]])] or [None]
+
+                                for vtime in sorted(valid_times):
+                                    if vtime and i_vtimes and vtime not in i_vtimes:
+                                        logging.warning(f"Requested vtime {vtime} at {itime} not present for "
+                                                        f"{dataset} {plot_object.name}! vtimes present: {i_vtimes}")
+                                        continue
+                                    elif not i_vtimes:
+                                        vtime = None
+
+                                    style = plot_object.styles[0][0] if plot_object.styles else None
+                                    kwargs = {"plot_object": plot_object,
+                                              "init_time": itime,
+                                              "valid_time": vtime}
+                                    filename = f"{l_type}_{dataset if multiple_datasets else ''}{plot_object.name}-" \
+                                               + f"Noneit{itime}vt{vtime}".replace(" ", "_").replace(":", "_")\
+                                                                          .replace("-", "_")
+
+                                    exists = os.path.exists(os.path.join(location, "plots", filename + ".png"))
+
+                                    if driver == self.lsec_drivers and not exists:
+                                        plot_driver.set_plot_parameters(**kwargs,
+                                                                        lsec_path=[[0, 0, 20000], [1, 1, 20000]],
+                                                                        lsec_numpoints=201,
+                                                                        lsec_path_connection="linear")
+                                        path = [[min(plot_driver.lat_data), min(plot_driver.lon_data), 20000],
+                                                [max(plot_driver.lat_data), max(plot_driver.lon_data), 20000]]
+                                        plot_driver.update_plot_parameters(lsec_path=path)
+
+                                    elif driver == self.vsec_drivers and not exists:
+                                        plot_driver.set_plot_parameters(**kwargs, vsec_path=[[0, 0], [1, 1]],
+                                                                        vsec_numpoints=201, figsize=[800, 600],
+                                                                        vsec_path_connection="linear", style=style,
+                                                                        noframe=False, bbox=[101, 1050, 10, 180])
+                                        path = [[min(plot_driver.lat_data), min(plot_driver.lon_data)],
+                                                [max(plot_driver.lat_data), max(plot_driver.lon_data)]]
+                                        plot_driver.update_plot_parameters(vsec_path=path)
+
+                                    elif driver == self.hsec_drivers:
+                                        elevations = plot_object.get_elevations()
+                                        elevation = float(elevations[len(elevations) // 2]) if len(elevations) > 0 \
+                                            else None
+                                        plot_driver.set_plot_parameters(**kwargs, noframe=False, figsize=[800, 600],
+                                                                        crs="EPSG:4326", style=style,
+                                                                        bbox=[-15, 35, 30, 65],
+                                                                        level=elevation)
+                                        bbox = [min(plot_driver.lon_data), min(plot_driver.lat_data),
+                                                max(plot_driver.lon_data), max(plot_driver.lat_data)]
+                                        vert_units = plot_driver.vert_units
+                                        plot_driver.update_plot_parameters(bbox=bbox)
+
+                                        # All specified levels, or the mid if empty, or all if "all",
+                                        # or None if there are no levels
+                                        rendered_levels = [float(elev) if elev else elev for elev in
+                                                           (levels.split(",") if (levels != "all" and levels != "") else
+                                                            elevations if levels == "all" else [elevation])] or [None]
+
+                                        for level in sorted(rendered_levels):
+                                            if level and elevations and level \
+                                                    not in [float(elev) for elev in elevations]:
+                                                logging.warning(f"Requested level {level} not present for "
+                                                                f"{dataset} {plot_object.name}! Levels present: "
+                                                                f"{elevations}")
+                                                continue
+                                            elif not elevations:
+                                                level = None
+
+                                            plot_driver.update_plot_parameters(level=level)
+
+                                            # filename is different for top layers, so needs a redefine
+                                            filename = f"{l_type}_{dataset if multiple_datasets else ''}" \
+                                                       f"{plot_object.name}-" + \
+                                                       f"{level}{vert_units}it{itime}vt{vtime}".replace(" ", "_")\
+                                                                                               .replace(":", "_")\
+                                                                                               .replace("-", "_")
+                                            exists = os.path.exists(os.path.join(location, "plots", filename + ".png"))
+
+                                            add_image(None if exists else plot_driver.plot(), plot_object,
+                                                      generate_code, sphinx, url_prefix=url_prefix,
+                                                      dataset=dataset if multiple_datasets else "", level=f"{level}" +
+                                                                                                f"{vert_units}",
+                                                      itime=str(itime), vtime=str(vtime), simple_naming=simple_naming)
+                                            if level:
+                                                add_levels([f"{level} {vert_units}"], vert_units)
+                                            add_times(itime, [vtime])
+                                        continue
+
+                                    add_image(None if exists else plot_driver.plot(), plot_object, generate_code,
+                                              sphinx, url_prefix=url_prefix,
+                                              dataset=dataset if multiple_datasets else "", itime=str(itime),
+                                              vtime=str(vtime), simple_naming=simple_naming)
+                                    add_times(itime, [vtime])
+
+                        except Exception as e:
+                            traceback.print_exc()
+                            logging.error("%s %s %s", plot_object.name, type(e), e)
+
+            write_html(sphinx)
+            if generate_code:
+                write_code_pages(sphinx, url_prefix)
+                if sphinx:
+                    write_doc_index()
+
     def register_hsec_layer(self, datasets, layer_class):
         """
         Register horizontal section layer in internal dict of layers.
@@ -262,7 +452,7 @@ class WMSServer(object):
                     raise ValueError("dataset '%s' not available", dataset)
             self.vsec_layer_registry[dataset][layer.name] = layer
 
-    def register_lsec_layer(self, datasets, variable=None, layer_class=None):
+    def register_lsec_layer(self, datasets, variable=None, filetype="ml", layer_class=None):
         """
         Register linear section layer in internal dict of layers.
 
@@ -274,7 +464,7 @@ class WMSServer(object):
         for dataset in datasets:
             try:
                 if variable:
-                    layer = layer_class(self.lsec_drivers[dataset], variable)
+                    layer = layer_class(self.lsec_drivers[dataset], variable, filetype)
                 else:
                     layer = layer_class(self.lsec_drivers[dataset])
             except KeyError as ex:
